<?php

// This file `jetpack_autoload_filemap.php` was auto generated by automattic/jetpack-autoloader.

$vendorDir = dirname(__DIR__);
$baseDir   = dirname($vendorDir);

return array(
	'a4a119a56e50fbb293281d9a48007e0e' => array(
		'version' => '1.30.0.0',
		'path'    => $vendorDir . '/symfony/polyfill-php80/bootstrap.php'
	),
	'3773ef3f09c37da5478d578e32b03a4b' => array(
		'version' => '2.3.0.0',
		'path'    => $vendorDir . '/automattic/jetpack-assets/actions.php'
	),
	'44e8414cd27982ecf86403af6e48f123' => array(
		'version' => '9.4.1.0',
		'path'    => $baseDir . '/src/StoreApi/deprecated.php'
	),
	'9ce98895d0a470c71998c4b530020d26' => array(
		'version' => '9.4.1.0',
		'path'    => $baseDir . '/src/StoreApi/functions.php'
	),
	'c379ea42c3f5964a973a7106b08c5ef0' => array(
		'version' => '9.4.1.0',
		'path'    => $baseDir . '/src/Blocks/Domain/Services/functions.php'
	),
);
