.tsdk-banner-cta {
    position: relative;
    display: inline-block;
}

.tsdk-banner-urgency-text {
    position: absolute;
    top: 6%;
    left: 1%;
    color: white;
    padding: 5px;
    font-size: 16px;
    z-index: 10;
    text-transform: uppercase;
    font-weight: 700;
}

.tsdk-banner-img {
    width: 100%;
    height: auto;
}

@media (max-width: 1100px) {
    .tsdk-banner-urgency-text {
        font-size: 10px;
        top: 0;
        left: 1%;
    }
}

@media (max-width: 950px) {
    .tsdk-banner-urgency-text {
        font-size: 10px;
        left: 1%;
    }
}

@media (max-width: 500px) {
    .tsdk-banner-urgency-text {
        font-size: 6px;
        top: -10%;
        left: 1%;
    }
}

@media (max-width: 420px) {
    .tsdk-banner-urgency-text {
        left: 0%;
    }
}

.notice:not(#tsdk_banner) {
    display: none;
}