<?php
return ['uskomaton','hämmästyttävä','ihmeellinen','ainutlaatuinen','kaunis','onnellisuus','loistava','kiehtova','karismaattinen','järkyttävä','selkeä','täysin','luottamuksellinen','luottamus','merkittävä','luova','ehdottomasti','herkullinen','todistaa','kiirehdi','päättäväinen','arvokas','dynaaminen','vaikuttava','välttämätön','inspiroiva','innovatiivinen','intensiivinen','tehokas','taianomainen','mahtava','historiallinen','tärkeä','uskomaton','välttämätön','unohtumaton','vastustamaton','legendaarinen','valoisa','ylellisyys','taianomainen','mahtava','majesteettinen','ikimuistoinen','ihmeellinen','ihmeellinen','motivoi','tarpeellinen','uusi','virallinen','täydellinen','intohimoinen','vakuuttava','ilmiömäinen','ilo','suosittu','voima','arvostettu','ihmeellinen','syvällinen','vauras','voimakas','laatu','säteilevä','nopea','menestys','vallankumouksellinen','tyytyväinen','turvallisuus','sensaatio','rauhallinen','ylellinen','loistava','jalo','yllättävä','lahjakas','pelottava','ainutlaatuinen','arvo','eloisa','voittoisa','elävä','todella','innostunut','aitoa','seikkailunhaluinen','upea','ainutlaatuinen','taattu','erityinen','fiksu','loistava','suuri','ilmainen','taitava','rajaton','virheetön','pettämätön','loputon','vaikutusvaltainen','nokkelia','ikimuistoinen','korvaamaton','johtaja','mestari','huomattava','uusi','uraauurtava','voimakas','tunnustettu','vallankumouksellinen','yllättävä','ylivoimainen','voittoisa','ultra','rohkea','arvokas','edistyksellinen','voimakas','näkemyksellinen','tahto','elintärkeä','voitto','kunniallinen','pysäyttämätön','verraton','älykäs','voittamaton','vapaus','ylpeys','rauha','edistys','uudistettu','viisaus','tyydytys','turvallinen','rauhallisuus','ylitys','lahjakkuus','ylittää','muutoksen','rohkeus','voitto'];
