<?php
return ['absolument','amazement','astonishing','authentique','beau','bien-être','brillant','captivant','charismatique','choc','clair','complètement','confidentiel','confiance','conséquent','créatif','définitivement','délicieux','démontrer','dépêchez-vous','déterminé','digne','dynamique','éblouissant','éclatant','économique','éfficace','élégant','émotionnel','énergique','énorme','époustouflant','essentiel','étonnant','exclusif','expérience','fabuleux','fantastique','formidable','fort','garanti','géant','généreux','grandiose','gratuit','habile','harmonieux','historique','hors pair','important','incroyable','indispensable','inoubliable','inspirant','innovant','intense','invention','irrésistible','légendaire','lumineux','luxe','magique','magnifique','majestueux','marquant','merveilleux','miraculeux','motivant','nécessaire','nouvelle','officiel','parfait','passionné','persuasif','phénoménal','plaisir','populaire','pouvoir','prestigieux','prodigieux','profond','prospère','puissant','qualité','radieux','rapide','réussi','révolutionnaire','satisfait','sécurité','sensationnel','serein','somptueux','splendide','sublime','surprenant','talentueux','terrifiant','unique','valeur','vibrant','victorieux','vif','vraiment','zélé'];
