<?php

// DO NOT EDIT THIS FILE - IT IS GENERATED BY constant_generator.php

namespace donatj\UserAgent;

interface Bots {

	const ADKERNELTOPICCRAWLER  = 'AdkernelTopicCrawler';
	const ADSBOT_GOOGLE         = 'AdsBot-Google';
	const AHREFSBOT             = 'AhrefsBot';
	const AHREFSSITEAUDIT       = 'AhrefsSiteAudit';
	const AMAZONADBOT           = 'AmazonAdBot';
	const AMAZONBOT             = 'Amazonbot';
	const APPLEBOT              = 'Applebot';
	const ARCHIVE_ORG_BOT       = 'archive.org_bot';
	const AWARIOBOT             = 'AwarioBot';
	const BAIDUSPIDER           = 'Baiduspider';
	const BAIDUSPIDER_RENDER    = 'Baiduspider-render';
	const BARKROWLER            = 'Barkrowler';
	const BINGBOT               = 'bingbot';
	const BLEXBOT               = 'BLEXBot';
	const BRAVEBOT              = 'Bravebot';
	const CENSYSINSPECT         = 'CensysInspect';
	const CHATGPT_USER          = 'ChatGPT-User';
	const CLAUDEBOT             = 'ClaudeBot';
	const COCCOCBOT_IMAGE       = 'coccocbot-image';
	const COCCOCBOT_WEB         = 'coccocbot-web';
	const CURL                  = 'curl';
	const DATAFORSEOBOT         = 'DataForSeoBot';
	const DISCORDBOT            = 'Discordbot';
	const DOTBOT                = 'DotBot';
	const DVBOT                 = 'DVbot';
	const EV_CRAWLER            = 'ev-crawler';
	const FACEBOOKEXTERNALHIT   = 'facebookexternalhit';
	const FEEDBOT               = 'feedbot';
	const FEEDVALIDATOR         = 'FeedValidator';
	const GOOGLEBOT             = 'Googlebot';
	const GOOGLEBOT_IMAGE       = 'Googlebot-Image';
	const GOOGLEBOT_VIDEO       = 'Googlebot-Video';
	const GOOGLE_READ_ALOUD     = 'Google-Read-Aloud';
	const GOOGLE_SAFETY         = 'Google-Safety';
	const GPTBOT                = 'GPTBot';
	const HEEXYBOT              = 'Heexybot';
	const HUBSPOT_DOMAIN_CHECK  = 'HubSpot Domain check';
	const IBOUBOT               = 'IbouBot';
	const IMESSAGEBOT           = 'iMessageBot';
	const LEIKIBOT              = 'Leikibot';
	const MASTODON              = 'Mastodon';
	const MINIFLUX              = 'Miniflux';
	const MODATSCANNER          = 'ModatScanner';
	const MOJEEKBOT             = 'MojeekBot';
	const MSNBOT_MEDIA          = 'msnbot-media';
	const OAI_SEARCHBOT         = 'OAI-SearchBot';
	const PERPLEXITYBOT         = 'PerplexityBot';
	const PERPLEXITY_USER       = 'Perplexity-User';
	const PETALBOT              = 'PetalBot';
	const PINTERESTBOT          = 'Pinterestbot';
	const PROXIMIC              = 'proximic';
	const SCOURRSSBOT           = 'ScourRSSBot';
	const SEMRUSHBOT            = 'SemrushBot';
	const SEMRUSHBOT_SI         = 'SemrushBot-SI';
	const SERANKINGBACKLINKSBOT = 'SERankingBacklinksBot';
	const SEZNAMBOT             = 'SeznamBot';
	const SLACKBOT              = 'Slackbot';
	const SMTBOT                = 'SMTBot';
	const SPIDERLING            = 'SpiderLing';
	const SURDOTLYBOT           = 'SurdotlyBot';
	const TELEGRAMBOT           = 'TelegramBot';
	const TWITTERBOT            = 'Twitterbot';
	const WELLKNOWNBOT          = 'WellKnownBot';
	const WGET                  = 'Wget';
	const WORDPRESS             = 'WordPress';
	const WPBOT                 = 'wpbot';
	const YANDEXBOT             = 'YandexBot';
	const YANDEXIMAGES          = 'YandexImages';
	const YANDEXMOBILEBOT       = 'YandexMobileBot';
	const YANDEXRCA             = 'YandexRCA';
	const YANDEXUSERPROXY       = 'YandexUserproxy';
	const YETI                  = 'Yeti';

}

