<?php

// DO NOT EDIT THIS FILE - IT IS GENERATED BY constant_generator.php

namespace donatj\UserAgent;

interface Platforms {

	const MACINTOSH        = 'Macintosh';
	const CHROME_OS        = 'Chrome OS';
	const LINUX            = 'Linux';
	const WINDOWS          = 'Windows';
	const ANDROID          = 'Android';
	const BLACKBERRY       = 'BlackBerry';
	const FREEBSD          = 'FreeBSD';
	const FUCHSIA          = 'Fuchsia';
	const IPAD             = 'iPad';
	const IPHONE           = 'iPhone';
	const IPOD             = 'iPod';
	const KINDLE           = 'Kindle';
	const KINDLE_FIRE      = 'Kindle Fire';
	const NETBSD           = 'NetBSD';
	const NEW_NINTENDO_3DS = 'New Nintendo 3DS';
	const NINTENDO_3DS     = 'Nintendo 3DS';
	const NINTENDO_DS      = 'Nintendo DS';
	const NINTENDO_SWITCH  = 'Nintendo Switch';
	const NINTENDO_WII     = 'Nintendo Wii';
	const NINTENDO_WIIU    = 'Nintendo WiiU';
	const OPENBSD          = 'OpenBSD';
	const PLAYBOOK         = 'PlayBook';
	const PLAYSTATION_3    = 'PlayStation 3';
	const PLAYSTATION_4    = 'PlayStation 4';
	const PLAYSTATION_5    = 'PlayStation 5';
	const PLAYSTATION_VITA = 'PlayStation Vita';
	const SAILFISH         = 'Sailfish';
	const SYMBIAN          = 'Symbian';
	const TIZEN            = 'Tizen';
	const WINDOWS_PHONE    = 'Windows Phone';
	const XBOX             = 'Xbox';
	const XBOX_ONE         = 'Xbox One';

}

