<?php
return ['increíble','asombroso','maravilloso','único','hermoso','felicidad','brillante','cautivador','carismático','impactante','claro','completamente','confidencial','confianza','significativo','creativo','definitivamente','delicioso','demostrar','apresúrate','decidido','digno','dinámico','impresionante','esencial','inspirador','innovador','intenso','eficaz','mágico','magnífico','histórico','importante','increíble','indispensable','inolvidable','irresistible','legendario','luminoso','lujo','mágico','magnífico','majestuoso','memorable','maravilloso','milagroso','motivador','necesario','nuevo','oficial','perfecto','apasionado','persuasivo','fenomenal','placer','popular','poder','prestigioso','prodigioso','profundo','próspero','poderoso','calidad','radiante','rápido','exitoso','revolucionario','satisfecho','seguridad','sensacional','sereno','suntuoso','espléndido','sublime','sorprendente','talentoso','terrorífico','único','valor','vibrante','victorioso','vivo','verdaderamente','celoso','auténtico','aventurero','espectacular','exclusivo','garantizado','extraordinario','fabuloso','fascinante','formidable','genial','grandioso','gratuito','hábil','ilimitado','impecable','infalible','infinitamente','influyente','ingenioso','inolvidable','irremplazable','líder','maestro','notable','novedoso','pionero','poderoso','reconocido','revolucionario','sorprendente','superior','triunfante','ultra','valiente','valioso','vanguardista','vigoroso','visionario','voluntad','vital','triunfo','glorioso','imparable','inigualable','inteligente','invencible','libertad','orgullo','paz','progreso','renovado','sabiduría','satisfacción','seguro','serenidad','superación','talento','transcendente','transformador','valentía','victoria'];
