<?php
if (!defined('TENWEB_SLUG')) {
    define('TENWEB_SLUG', "10web-manager/10web-manager.php");
}
if (!defined('TENWEB_LANG')) {
    define('TENWEB_LANG', "tenweb");
}

if (!defined('TENWEBIO_PREFIX')) {
    define('TENWEBIO_PREFIX', 'iowd');
}
if (!defined('TENWEBIO_MANAGER_PREFIX')) {
    define('TENWEBIO_MANAGER_PREFIX', 'tenweb');
}

if (!defined('TENWEB_PREFIX')) {
    define('TENWEB_PREFIX', 'tenweb');
}

if(!defined('TENWEB_CONNECTED_MANAGER')){
    define('TENWEB_CONNECTED_MANAGER','manager');
}

if(!defined('TENWEB_CONNECTED_SPEED')){
    define('TENWEB_CONNECTED_SPEED','speed_optimizer');
}

if(!defined('TENWEB_SO_IN_PROGRESS_LOCK')){
    define('TENWEB_SO_IN_PROGRESS_LOCK', 300);
}


