<?php
return ['すごい','感動','驚く','唯一無二','美しい','幸福','輝かしい','魅惑的','カリスマ的','衝撃','明瞭','完全に','信頼できる','自信','重要','創造的','決定的に','美味しい','証明する','急いで','決意','価値がある','ダイナミック','感動的','不可欠','インスピレーション','革新的','強力な','魔法のような','壮大','歴史的','重要な','信じられない','絶対必要','忘れられない','魅力的','刺激的','効果的','革命的','成功','喜び','安全','感動的','満ち足りた','素晴らしい','独特','価値がある','素晴らしい','熱心な','勝利','活気づける','真に','情熱的','熱心な','優雅','大胆','理想的','信じられないほど','現実的','最適','並外れた','豊富','壮観','夢のような','感激','魅了する','完璧','壮大','豪華','驚異的','意欲的','熱狂的','画期的','強烈','驚愕','緊急','解決','最高','豊か','大胆不敵','有名','愛される','特別','信じられる','確実','情熱','期待','最前線','開拓者','パワフル','影響力','高品質','偉大','簡単','安心','重要','独自','栄光','無敵','自由','希望','奇跡','成功','豊かな','力強い','栄光','飛躍','注目','壮大','挑戦的','伝説','無敵','華麗','驚異','素晴らしい','壮麗','偉大な','偉大','伝説的','刺激的','エキサイティング','期待','憧れ','優秀','絶対','大成功','想像を絶する','完璧','魅力的','感動的','偉大','優雅','素晴らしい','革新的','圧倒的','唯一'];
