<?php return [
	'akta', 'ansedd', 'ansett', 'ansedda', 'attraktiv', 'attraktivare', 'attraktivast', 'attraktivt',
	'banbrytande', 'belaten', 'belåten', 'belåtnare', 'belåtnast', 'belåtet', 'beromd', 'berömd', 'berömda',
	'berömdare', 'berömdast', 'berömt', 'betydelsefull', 'betydelsefullare', 'betydelsefullast', 'betydelsefullt',
	'bevisad', 'bevisa', 'bevisade', 'bevisar', 'bevisat', 'blandande', 'blanda', 'blandar', 'blandade', 'blandat',
	'briljant', 'briljantare', 'briljantast', 'effektiv', 'effektivare', 'effektivast', 'effektivt', 'eftertraktad',
	'eftertraktade', 'eftertraktar', 'eftertraktat', 'elegant', 'elegantare', 'elegantast', 'enastaende',
	'enastående', 'energisk', 'energiskare', 'energiskt', 'exklusiv', 'exklusivare', 'exklusivast', 'exklusivt',
	'fangslande', 'fantastisk', 'fantastiska', 'fantastiskt', 'fint', 'fin', 'finare', 'finast', 'finaste',
	'framgangsrik', 'framgångsrik', 'framgångsrikare', 'framgångsrikast', 'framgångsrikt', 'fullandad',
	'fulländad', 'fulländade', 'fulländat', 'fullgora', 'fullgjorda', 'fullgjorde', 'fullgjort', 'fängslande',
	'fängslad', 'fängslade', 'fängslat', 'fängslar', 'förändra', 'förändrat', 'förträfflig', 'förträffligare',
	'förträffligast', 'förträffligt', 'glansande', 'glänsa', 'glänsande', 'glänste', 'glänst', 'glomd', 'glömda',
	'glömde', 'glömma', 'glömmer', 'glömt', 'hyllad', 'hyllade', 'hyllar', 'hyllat', 'imponerande', 'imponerad',
	'imponerade', 'imponerar', 'imponerat', 'inspirerad', 'inspirerande', 'inspirerar', 'inspirerade', 'inspirerat',
	'kand', 'känd', 'kända', 'kändare', 'kändast', 'känt', 'kraftfull', 'kraftfullare', 'kraftfullast', 'kraftfullt',
	'kraftig', 'kraftigare', 'kraftigast', 'kraftigaste', 'kreativ', 'kreativare', 'kreativast', 'kreativt', 'langtan',
	'legendarisk', 'legendariska', 'legendariskt', 'lidelsefull', 'lidelsefulla', 'lidelsefullare', 'lidelsefullast',
	'lidelsefullt', 'livlig', 'livligare', 'livligast', 'livligt', 'längtan', 'lysande', 'lyser', 'lyste', 'maktig',
	'mäktig', 'mäktigare', 'mäktigast', 'mäktigt', 'minnesvard', 'minnesvärd', 'minnesvärda', 'minnesvärt', 'oerhord',
	'oerhörd', 'oerhördare', 'oerhördast', 'oerhört', 'oforglomlig', 'oförglömlig', 'oförglömligare',
	'oförglömligast', 'oförglömligt', 'omtycka', 'omtycker', 'omtyckte', 'omtyckt', 'omtyckta', 'ondskefull',
	'ondskefullt', 'ondska', 'onskan', 'otroligare', 'otrolig', 'otroligast', 'otroligt', 'palitlig', 'passion',
	'passionerad', 'passionerade', 'passionerat', 'praktfull', 'praktfullare', 'praktfullast', 'praktfullt', 'pris',
	'prisa', 'prisade', 'prisar', 'prisa', 'prisar', 'prisat', 'professionell', 'professionella', 'professionellt',
	'pålitlig', 'pålitligare', 'pålitligast', 'pålitligt', 'renommerad', 'renommerade', 'renommerat', 'revolution',
	'revolutionera', 'saker', 'säker', 'säkrare', 'säkrast', 'säkert', 'sallsynt', 'sällsynt', 'sällsyntare',
	'sällsyntast', 'sällsynthet', 'sensationell', 'sensationellare', 'sensationellast', 'sensationellt', 'skapad',
	'skapade', 'skapande', 'skapar', 'skapat', 'sofistikerad', 'sofistikerade', 'sofistikerat', 'speciell',
	'speciellare', 'speciellast', 'speciella', 'speciellt', 'stark', 'starkare', 'starkast', 'starkt', 'storslagen',
	'storslagna', 'storslaget', 'stralande', 'stråla', 'strålade', 'strålar', 'strålat', 'strålande', 'tillfredstalld',
	'tillfredsställd', 'tillfredsställda', 'tillfredsställde', 'tillfredsställer', 'tillfredsställa',
	'tillfredsställt', 'trygg', 'tryggare', 'tryggast', 'tryggt', 'underbar', 'underbarare', 'underbarast', 'underbart',
	'ungdomlig', 'ungdomligare', 'ungdomligast', 'ungdomligt', 'uppfylla', 'uppfylld', 'uppfyller', 'uppfyllt',
	'uppriktig', 'uppriktigare', 'uppriktigast', 'uppriktigt', 'utmarkt', 'utmärka', 'utmärkt', 'utmärktast',
	'utmärker', 'utmärkte', 'utstralad', 'utstrålade', 'utstrålar', 'utstrålat', 'utstrålande', 'vasentlig',
	'väsentlig', 'väsentligare', 'väsentligast', 'väsentligt', 'viktig', 'viktigare', 'viktigast', 'viktigt', 'äkta',
	'äktare', 'äktast', 'äktaste', 'önskad', 'önska', 'önskan', 'överlägsen', 'överlägsnare', 'överlägsnast',
	'överlägset'
];