<?php

// autoload_files.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'f069176919c5ac16d5433c9f6edc890b' => $vendorDir . '/codeinwp/themeisle-sdk/load.php',
);
