<?php
return [ 'elképesztő', 'megdöbbentő', 'szenzációs', 'meglepő', 'elragadó', 'félelmetes', 'ijesztő', 'vonzó', 'ellentmondásos', 'zseniális', 'borzalmas', 'remek', 'elkápráztató', 'katasztrofális', 'meghökkentő', 'lenyűgöző', 'csodálatos', 'ijedt', 'megnyugtató', 'felkavaró', 'inspiráló', 'lehangoló', 'fájdalmas', 'gyönyörű', 'bámulatos', 'megszédítő', 'bűnös', 'ördögi', 'lebilincselő', 'megsemmisítő', 'látványos', 'hátborzongató', 'csábító', 'fényűző', 'szívszorító', 'tragikus', 'szertelen', 'ölelő', 'megbocsátó', 'öntelt', 'önző', 'buzgó', 'hajszolt', 'könyörtelen', 'elmés', 'fantasztikus', 'gátlástalan', 'őrjítő', 'visszataszító', 'féktelen', 'összetört', 'megrontott', 'fényes', 'váratlan', 'élénk', 'kifinomult', 'döbbenetes', 'felüdítő', 'megváltó', 'megtévesztő', 'hatékony', 'érzelmes', 'életerős', 'megható', 'haragos', 'elégedett', 'bosszantó', 'nagylelkű', 'ragyogó', 'izgalmas', 'csalódott', 'megrázó', 'méltóságteljes', 'csábos', 'befogadó', 'kirekesztő', 'lelkes', 'elkésett', 'megdönthetetlen','csodálkozó', 'feszült', 'dermesztő', 'forró', 'összetört', 'élvezetes', 'nyomasztó', 'diadalmas', 'megsebzett', 'megtorló', 'kitartó', 'reménytelen', 'érzékeny', 'tűnődő', 'mágikus', 'kifürkészhetetlen', 'vágyakozó', 'ütős', 'megbékélő', 'gyászos' ];
