<?php

// DO NOT EDIT THIS FILE - IT IS GENERATED BY constant_generator.php

namespace donatj\UserAgent;

interface Browsers {

	const ADSBOT_GOOGLE       = 'AdsBot-Google';
	const ANDROID_BROWSER     = 'Android Browser';
	const APPLEBOT            = 'Applebot';
	const BAIDUSPIDER         = 'Baiduspider';
	const BINGBOT             = 'bingbot';
	const BLACKBERRY_BROWSER  = 'BlackBerry Browser';
	const BROWSER             = 'Browser';
	const BUNJALLOO           = 'Bunjalloo';
	const CAMINO              = 'Camino';
	const CHATGPT_USER        = 'ChatGPT-User';
	const CHROME              = 'Chrome';
	const CURL                = 'curl';
	const EDGE                = 'Edge';
	const FACEBOOKEXTERNALHIT = 'facebookexternalhit';
	const FEEDVALIDATOR       = 'FeedValidator';
	const FIREFOX             = 'Firefox';
	const GOOGLEBOT           = 'Googlebot';
	const GOOGLEBOT_IMAGE     = 'Googlebot-Image';
	const GOOGLEBOT_VIDEO     = 'Googlebot-Video';
	const GPTBOT              = 'GPTBot';
	const HEADLESSCHROME      = 'HeadlessChrome';
	const IEMOBILE            = 'IEMobile';
	const IMESSAGEBOT         = 'iMessageBot';
	const KINDLE              = 'Kindle';
	const LYNX                = 'Lynx';
	const MASTODON            = 'Mastodon';
	const MIDORI              = 'Midori';
	const MIUIBROWSER         = 'MiuiBrowser';
	const MSIE                = 'MSIE';
	const MSNBOT_MEDIA        = 'msnbot-media';
	const NETFRONT            = 'NetFront';
	const NINTENDOBROWSER     = 'NintendoBrowser';
	const OAI_SEARCHBOT       = 'OAI-SearchBot';
	const OCULUSBROWSER       = 'OculusBrowser';
	const OPERA               = 'Opera';
	const PUFFIN              = 'Puffin';
	const SAFARI              = 'Safari';
	const SAILFISHBROWSER     = 'SailfishBrowser';
	const SAMSUNGBROWSER      = 'SamsungBrowser';
	const SILK                = 'Silk';
	const SLACKBOT            = 'Slackbot';
	const TELEGRAMBOT         = 'TelegramBot';
	const TIZENBROWSER        = 'TizenBrowser';
	const TWITTERBOT          = 'Twitterbot';
	const UC_BROWSER          = 'UC Browser';
	const VALVE_STEAM_TENFOOT = 'Valve Steam Tenfoot';
	const VIVALDI             = 'Vivaldi';
	const WGET                = 'Wget';
	const WHALE               = 'Whale';
	const WORDPRESS           = 'WordPress';
	const YANDEX              = 'Yandex';
	const YANDEXBOT           = 'YandexBot';
	const YETI                = 'Yeti';

}

