#ti-sdk-float-widget .ti-float-button-enter{opacity:.01}#ti-sdk-float-widget .ti-float-button-enter.ti-float-button-enter-active{opacity:1;transition:opacity 200ms ease-in}#ti-sdk-float-widget .ti-float-button-leave{opacity:1}#ti-sdk-float-widget .ti-float-button-leave.ti-float-button-leave-active{opacity:.01;transition:opacity 200ms ease-in}#ti-sdk-float-widget .ti-float-button{-webkit-appearance:none;-moz-appearance:none;appearance:none;border:0;padding:0;width:51px;height:51px;background-color:#f0f0f1;border-radius:50%;display:flex;align-items:center;justify-content:center;position:fixed;bottom:32px;right:24px}#ti-sdk-float-widget .ti-float-button:hover,#ti-sdk-float-widget .ti-float-button:focus{cursor:pointer}#ti-sdk-float-widget .ti-float-button>img,#ti-sdk-float-widget .ti-float-button>span{position:absolute;opacity:0;transition:opacity 200ms ease-in}#ti-sdk-float-widget .ti-float-button>img.active,#ti-sdk-float-widget .ti-float-button>span.active{opacity:1}#ti-sdk-float-widget .ti-float-button>img.active:hover,#ti-sdk-float-widget .ti-float-button>img.active:focus,#ti-sdk-float-widget .ti-float-button>span.active:hover,#ti-sdk-float-widget .ti-float-button>span.active:focus{opacity:.9}#ti-sdk-float-widget .ti-float-button>span{--ti-icon-size: 42px;font-size:var(--ti-icon-size);width:var(--ti-icon-size);height:var(--ti-icon-size);border-radius:50%;color:#fff;background-color:var(--ti-float-primary-color, #2271b1)}#ti-sdk-float-widget .ti-float-button>img{width:42px;height:42px}.ti-float-widget-panel{width:275px;border-radius:8px;background-color:#fff;box-shadow:0px 2px 15px 0px rgba(0,0,0,.1);margin:12px}.ti-float-widget-panel .ti-float-widget-panel__header{border-radius:8px 8px 0 0;padding:12px 12px 48px 12px;background:linear-gradient(75deg, var(--ti-float-primary-color, #2271b1) 15%, rgba(255, 255, 255, 0.65) 500%);display:flex;flex-direction:column;align-items:flex-start;gap:8px}.ti-float-widget-panel .ti-float-widget-panel__header img{background:#fff;border-radius:50%;width:42px;height:42px;box-shadow:0 0 0px 5px #fff;margin:5px}.ti-float-widget-panel .ti-float-widget-panel__header p,.ti-float-widget-panel .ti-float-widget-panel__header h3{color:#fff}.ti-float-widget-panel .ti-float-widget-panel__header p{font-size:14px}.ti-float-widget-panel .ti-float-widget-panel__header h3{font-size:20px;margin:0}.ti-float-widget-panel .ti-float-widget-panel__content{padding:12px;display:flex;flex-direction:column;gap:8px;margin-top:-36px}.ti-float-widget-panel .ti-float-widget-panel__content>a{display:flex;gap:8px;align-items:center;justify-content:flex-start;background:linear-gradient(-75deg, rgb(255, 255, 255) 0%, rgb(255, 255, 255) 100%);padding:12px;border-radius:8px;box-shadow:0px 2px 15px 0px rgba(0,0,0,.15);color:#000;font-size:14px;text-decoration:none;transition:background 200ms ease-in}.ti-float-widget-panel .ti-float-widget-panel__content>a:hover{background:linear-gradient(-75deg, var(--ti-float-primary-color, #2271b1) -150%, rgb(255, 255, 255) 50%)}.ti-float-widget-panel .ti-float-widget-panel__content>a:focus-visible{box-shadow:0 0 0 2px var(--ti-float-primary-color, #2271b1)}.ti-float-widget-panel .ti-float-widget-panel__content>a .dashicons{background:#2271b1;color:#efefef;font-size:16px;width:24px;height:24px;padding:4px;border-radius:4px}.ti-float-widget-panel .ti-float-widget-panel__content>a .dashicons.dashicons-book-alt{background:#fdd7ae;color:#ff9d32}.ti-float-widget-panel .ti-float-widget-panel__content>a .dashicons.dashicons-format-status{background:#96d7c7;color:#30955e}.ti-float-widget-panel .ti-float-widget-panel__content>a .dashicons.dashicons-admin-tools{background:#dac8ff;color:#783ff2}.ti-float-widget-panel .ti-float-widget-panel__content>a .dashicons.dashicons-superhero-alt{background:#ffe39e;color:#fdb600}.ti-float-widget-panel .ti-float-widget-panel__content>a .dashicons.dashicons-star-filled{background:#bfd0ff;color:#3068ff}
