/*!
 * CMB2 - v2.12.0 - 2026-05-31
 * https://cmb2.io
 * Copyright (c) 2026
 * Licensed GPLv2+
 */

/*--------------------------------------------------------------
 * Main Wrap
--------------------------------------------------------------*/
.cmb2-wrap {
  margin: 0;
}
.cmb2-wrap input,
.cmb2-wrap textarea {
  max-width: 100%;
}
.cmb2-wrap input[type=text].cmb2-oembed {
  width: 100%;
}
.cmb2-wrap textarea {
  width: 500px;
  padding: 8px;
}
.cmb2-wrap textarea.cmb2-textarea-code {
  font-family: "Courier 10 Pitch", Courier, monospace;
  line-height: 16px;
}
.cmb2-wrap input.cmb2-text-small, .cmb2-wrap input.cmb2-timepicker {
  width: 100px;
}
.cmb2-wrap input.cmb2-text-money {
  width: 90px;
}
.cmb2-wrap input.cmb2-text-medium {
  width: 230px;
}
.cmb2-wrap input.cmb2-upload-file {
  width: 65%;
}
.cmb2-wrap input.ed_button {
  padding: 2px 4px;
}
.cmb2-wrap input:not([type=hidden]) + input,
.cmb2-wrap input:not([type=hidden]) + .button-secondary,
.cmb2-wrap input:not([type=hidden]) + select {
  margin-left: 20px;
}
.cmb2-wrap ul {
  margin: 0;
}
.cmb2-wrap li {
  font-size: 14px;
  line-height: 16px;
  margin: 1px 0 5px 0;
}
.cmb2-wrap select {
  font-size: 14px;
  margin-top: 3px;
}
.cmb2-wrap input[type=radio] {
  margin: 0 5px 0 0;
  padding: 0;
}
.cmb2-wrap input[type=checkbox] {
  margin: 0 5px 0 0;
  padding: 0;
}
.cmb2-wrap button,
.cmb2-wrap .button-secondary {
  white-space: nowrap;
}
.cmb2-wrap .mceLayout {
  border: 1px solid #e9e9e9 !important;
}
.cmb2-wrap .mceIframeContainer {
  background: #ffffff;
}
.cmb2-wrap .meta_mce {
  width: 97%;
}
.cmb2-wrap .meta_mce textarea {
  width: 100%;
}
.cmb2-wrap .cmb-multicheck-toggle {
  margin-top: -1em;
}
.cmb2-wrap .wp-picker-clear.button,
.cmb2-wrap .wp-picker-default.button {
  margin-left: 6px;
  padding: 2px 8px;
}
.cmb2-wrap .cmb-row {
  margin: 0;
}
.cmb2-wrap .cmb-row:after {
  content: "";
  clear: both;
  display: block;
  width: 100%;
}
.cmb2-wrap .cmb-row.cmb-repeat .cmb2-metabox-description {
  padding-top: 0;
  padding-bottom: 1em;
}

body.block-editor-page.branch-5-3 .cmb2-wrap .cmb-row .cmb2-radio-list input[type=radio]::before {
  margin: 0.1875rem;
}
@media screen and (max-width: 782px) {
  body.block-editor-page.branch-5-3 .cmb2-wrap .cmb-row .cmb2-radio-list input[type=radio]::before {
    margin: 0.4375rem;
  }
}

.cmb2-metabox {
  clear: both;
  margin: 0;
}
.cmb2-metabox > .cmb-row:first-of-type > .cmb-td,
.cmb2-metabox > .cmb-row:first-of-type > .cmb-th,
.cmb2-metabox .cmb-field-list > .cmb-row:first-of-type > .cmb-td,
.cmb2-metabox .cmb-field-list > .cmb-row:first-of-type > .cmb-th {
  border: 0;
}

.cmb-add-row {
  margin: 1.8em 0 0;
}

.cmb-nested .cmb-td,
.cmb-repeatable-group .cmb-th,
.cmb-repeatable-group:first-of-type {
  border: 0;
}

.cmb-row:last-of-type,
.cmb2-wrap .cmb-row:last-of-type,
.cmb-repeatable-group:last-of-type {
  border-bottom: 0;
}

.cmb-repeatable-grouping {
  border: 1px solid #e9e9e9;
  padding: 0 1em;
}
.cmb-repeatable-grouping.cmb-row {
  margin: 0 0 0.8em;
}
.cmb-th {
  color: #222222;
  float: left;
  font-weight: 600;
  padding: 20px 10px 20px 0;
  vertical-align: top;
  width: 200px;
}
@media (max-width: 450px) {
  .cmb-th {
    font-size: 1.2em;
    display: block;
    float: none;
    padding-bottom: 1em;
    text-align: left;
    width: 100%;
  }
  .cmb-th label {
    display: block;
    margin-top: 0;
    margin-bottom: 0.5em;
  }
}

.cmb-td {
  line-height: 1.3;
  max-width: 100%;
  padding: 15px 10px;
  vertical-align: middle;
}

.cmb-type-title .cmb-td {
  padding: 0;
}

.cmb-th label {
  display: block;
  padding: 5px 0;
}

.cmb-th + .cmb-td {
  float: left;
}

.cmb-td .cmb-td {
  padding-bottom: 1em;
}

.cmb-remove-row {
  text-align: right;
}

.empty-row.hidden {
  display: none;
}

.cmb-repeat-table {
  background-color: #fafafa;
  border: 1px solid rgb(225.35, 225.35, 225.35);
}
.cmb-repeat-table .cmb-row.cmb-repeat-row {
  position: relative;
  counter-increment: el;
  margin: 0;
  padding: 10px 10px 10px 50px;
  border-bottom: none !important;
}
.cmb-repeat-table .cmb-row.cmb-repeat-row + .cmb-repeat-row {
  border-top: solid 1px #e9e9e9;
}
.cmb-repeat-table .cmb-row.cmb-repeat-row.ui-sortable-helper {
  outline: dashed 2px #e9e9e9 !important;
}
.cmb-repeat-table .cmb-row.cmb-repeat-row:before {
  content: counter(el);
  display: block;
  top: 0;
  left: 0;
  position: absolute;
  width: 35px;
  height: 100%;
  line-height: 35px;
  cursor: move;
  color: #757575;
  text-align: center;
  border-right: solid 1px #e9e9e9;
}
.cmb-repeat-table .cmb-row.cmb-repeat-row .cmb-td {
  margin: 0;
  padding: 0;
}
.cmb-repeat-table + .cmb-add-row {
  margin: 0;
}
.cmb-repeat-table + .cmb-add-row:before {
  content: "";
  width: 1px;
  height: 1.6em;
  display: block;
  margin-left: 17px;
  background-color: rgb(220.25, 220.25, 220.25);
}
.cmb-repeat-table .cmb-remove-row {
  top: 7px;
  right: 7px;
  position: absolute;
  width: auto;
  margin-left: 0;
  padding: 0 !important;
  display: none;
}
.cmb-repeat-table .cmb-remove-row > .cmb-remove-row-button {
  font-size: 20px;
  text-indent: -1000px;
  overflow: hidden;
  position: relative;
  height: auto;
  line-height: 1;
  padding: 0 10px 0;
}
.cmb-repeat-table .cmb-remove-row > .cmb-remove-row-button:before {
  content: "\f335";
  font-family: "Dashicons";
  speak: none;
  font-weight: normal;
  font-variant: normal;
  text-transform: none;
  line-height: 1;
  -webkit-font-smoothing: antialiased;
  margin: 0;
  text-indent: 0;
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  text-align: center;
  line-height: 1.3;
}
.cmb-repeat-table .cmb-repeat-row:hover .cmb-remove-row {
  display: block;
}

.cmb-repeatable-group .cmb-th {
  padding: 5px;
}
.cmb-repeatable-group .cmb-group-title {
  background-color: #e9e9e9;
  padding: 8px 12px 8px 2.2em;
  margin: 0 -1em;
  min-height: 1.5em;
  font-size: 14px;
  line-height: 1.4;
}
.cmb-repeatable-group .cmb-group-title h4 {
  border: 0;
  margin: 0;
  font-size: 1.2em;
  font-weight: 500;
  padding: 0.5em 0.75em;
}
.cmb-repeatable-group .cmb-group-title .cmb-th {
  display: block;
  width: 100%;
}
.cmb-repeatable-group .cmb-group-description .cmb-th {
  font-size: 1.2em;
  display: block;
  float: none;
  padding-bottom: 1em;
  text-align: left;
  width: 100%;
}
.cmb-repeatable-group .cmb-group-description .cmb-th label {
  display: block;
  margin-top: 0;
  margin-bottom: 0.5em;
}
.cmb-repeatable-group .cmb-shift-rows {
  margin-right: 1em;
}
.cmb-repeatable-group .cmb-shift-rows .dashicons-arrow-up-alt2 {
  margin-top: 0.15em;
}
.cmb-repeatable-group .cmb-shift-rows .dashicons-arrow-down-alt2 {
  margin-top: 0.2em;
}
.cmb-repeatable-group .cmb2-upload-button {
  float: right;
}

p.cmb2-metabox-description {
  color: #666;
  letter-spacing: 0.01em;
  margin: 0;
  padding-top: 0.5em;
}

span.cmb2-metabox-description {
  color: #666;
  letter-spacing: 0.01em;
}

.cmb2-metabox-title {
  margin: 0 0 5px 0;
  padding: 5px 0 0 0;
  font-size: 14px;
}

.cmb-inline ul {
  padding: 4px 0 0 0;
}

.cmb-inline li {
  display: inline-block;
  padding-right: 18px;
}

.cmb-type-textarea-code pre {
  margin: 0;
}

.cmb2-media-status .img-status {
  clear: none;
  display: inline-block;
  vertical-align: middle;
  margin-right: 10px;
  width: auto;
}
.cmb2-media-status .img-status img {
  max-width: 350px;
  height: auto;
}
.cmb2-media-status .img-status img,
.cmb2-media-status .embed-status {
  background: #eee;
  border: 5px solid #ffffff;
  outline: 1px solid #e9e9e9;
  box-shadow: inset 0 0 15px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(0, 0, 0, 0.05);
  background-image: linear-gradient(45deg, #d0d0d0 25%, transparent 25%, transparent 75%, #d0d0d0 75%, #d0d0d0), linear-gradient(45deg, #d0d0d0 25%, transparent 25%, transparent 75%, #d0d0d0 75%, #d0d0d0);
  background-position: 0 0, 10px 10px;
  background-size: 20px 20px;
  border-radius: 2px;
  -moz-border-radius: 2px;
  margin: 15px 0 0 0;
}
.cmb2-media-status .embed-status {
  float: left;
  max-width: 800px;
}
.cmb2-media-status .img-status, .cmb2-media-status .embed-status {
  position: relative;
}
.cmb2-media-status .img-status .cmb2-remove-file-button, .cmb2-media-status .embed-status .cmb2-remove-file-button {
  background: url(../images/ico-delete.png);
  height: 16px;
  left: -5px;
  position: absolute;
  text-indent: -9999px;
  top: -5px;
  width: 16px;
}
.cmb2-media-status .img-status .cmb2-remove-file-button {
  top: 10px;
}
.cmb2-media-status .img-status img, .cmb2-media-status .file-status > span {
  cursor: pointer;
}
.cmb2-media-status.cmb-attach-list .img-status img, .cmb2-media-status.cmb-attach-list .file-status > span {
  cursor: move;
}

.cmb-type-file-list .cmb2-media-status .img-status {
  clear: none;
  vertical-align: middle;
  width: auto;
  margin-right: 10px;
  margin-bottom: 10px;
  margin-top: 0;
}

.cmb-attach-list li {
  clear: both;
  display: inline-block;
  width: 100%;
  margin-top: 5px;
  margin-bottom: 10px;
}
.cmb-attach-list li img {
  float: left;
  margin-right: 10px;
}

.cmb2-remove-wrapper {
  margin: 0;
}

.child-cmb2 .cmb-th {
  text-align: left;
}

.cmb2-indented-hierarchy {
  padding-left: 1.5em;
}

@media (max-width: 450px) {
  .cmb-th,
  .cmb-td,
  .cmb-th + .cmb-td {
    display: block;
    float: none;
    width: 100%;
  }
}
/*--------------------------------------------------------------
 * Post Metaboxes
--------------------------------------------------------------*/
#poststuff .cmb-group-title {
  margin-left: -1em;
  margin-right: -1em;
  min-height: 1.5em;
}

#poststuff .repeatable .cmb-group-title {
  padding-left: 2.2em;
}

.cmb2-postbox .cmb2-wrap, .cmb-type-group .cmb2-wrap {
  margin: 0;
}
.cmb2-postbox .cmb2-wrap > .cmb-field-list > .cmb-row, .cmb-type-group .cmb2-wrap > .cmb-field-list > .cmb-row {
  padding: 1.8em 0;
}
.cmb2-postbox .cmb2-wrap input[type=text].cmb2-oembed, .cmb-type-group .cmb2-wrap input[type=text].cmb2-oembed {
  width: 100%;
}
.cmb2-postbox .cmb-row, .cmb-type-group .cmb-row {
  padding: 0 0 1.8em;
  margin: 0 0 0.8em;
}
.cmb2-postbox .cmb-row .cmbhandle, .cmb-type-group .cmb-row .cmbhandle {
  right: -1em;
  position: relative;
  color: #222222;
}
.cmb2-postbox .cmb-repeatable-grouping, .cmb-type-group .cmb-repeatable-grouping {
  padding: 0 1em;
  max-width: 100%;
  min-width: 1px !important;
}
.cmb2-postbox .cmb-repeatable-group > .cmb-row, .cmb-type-group .cmb-repeatable-group > .cmb-row {
  padding-bottom: 0;
}
.cmb2-postbox .cmb-th, .cmb-type-group .cmb-th {
  width: 18%;
  padding: 0 2% 0 0;
}
.cmb2-postbox .cmb-td, .cmb-type-group .cmb-td {
  margin-bottom: 0;
  padding: 0;
  line-height: 1.3;
}
.cmb2-postbox .cmb-th + .cmb-td, .cmb-type-group .cmb-th + .cmb-td {
  width: 80%;
  float: right;
}
.cmb2-postbox .cmb-row:not(:last-of-type),
.cmb2-postbox .cmb-repeatable-group:not(:last-of-type), .cmb-type-group .cmb-row:not(:last-of-type),
.cmb-type-group .cmb-repeatable-group:not(:last-of-type) {
  border-bottom: 1px solid #e9e9e9;
}
@media (max-width: 450px) {
  .cmb2-postbox .cmb-row:not(:last-of-type),
  .cmb2-postbox .cmb-repeatable-group:not(:last-of-type), .cmb-type-group .cmb-row:not(:last-of-type),
  .cmb-type-group .cmb-repeatable-group:not(:last-of-type) {
    border-bottom: 0;
  }
}
.cmb2-postbox .cmb-repeat-group-field,
.cmb2-postbox .cmb-remove-field-row, .cmb-type-group .cmb-repeat-group-field,
.cmb-type-group .cmb-remove-field-row {
  padding-top: 1.8em;
}
.cmb2-postbox .button-secondary .dashicons, .cmb-type-group .button-secondary .dashicons {
  line-height: 1.3;
}
.cmb2-postbox .button-secondary.move-up .dashicons, .cmb2-postbox .button-secondary.move-down .dashicons, .cmb-type-group .button-secondary.move-up .dashicons, .cmb-type-group .button-secondary.move-down .dashicons {
  line-height: 1.1;
}
.cmb2-postbox .button-secondary.cmb-add-group-row .dashicons, .cmb-type-group .button-secondary.cmb-add-group-row .dashicons {
  line-height: 1.5;
}

/*--------------------------------------------------------------
 * Context Metaboxes
--------------------------------------------------------------*/
/* Metabox collapse arrow indicators */
.js .cmb2-postbox.context-box .handlediv {
  text-align: center;
}
.js .cmb2-postbox.context-box .toggle-indicator:before {
  content: "\f142";
  display: inline-block;
  font: normal 20px/1 dashicons;
  speak: none;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  text-decoration: none !important;
}
.js .cmb2-postbox.context-box.closed .toggle-indicator:before {
  content: "\f140";
}

.cmb2-postbox.context-box {
  margin-bottom: 10px;
}
.cmb2-postbox.context-box.context-before_permalink-box {
  margin-top: 10px;
}
.cmb2-postbox.context-box.context-after_title-box {
  margin-top: 10px;
}
.cmb2-postbox.context-box.context-after_editor-box {
  margin-top: 20px;
  margin-bottom: 0;
}
.cmb2-postbox.context-box.context-form_top-box {
  margin-top: 10px;
}
.cmb2-postbox.context-box.context-form_top-box .hndle {
  font-size: 14px;
  padding: 8px 12px;
  margin: 0;
  line-height: 1.4;
}
.cmb2-postbox.context-box .hndle {
  cursor: auto;
}

.cmb2-context-wrap {
  margin-top: 10px;
}
.cmb2-context-wrap.cmb2-context-wrap-form_top {
  margin-right: 300px;
  width: auto;
}
.cmb2-context-wrap.cmb2-context-wrap-no-title .cmb2-metabox {
  padding: 10px;
}
.cmb2-context-wrap .cmb-th {
  padding: 0 2% 0 0;
  width: 18%;
}
.cmb2-context-wrap .cmb-td {
  width: 80%;
  padding: 0;
}
.cmb2-context-wrap .cmb-row {
  margin-bottom: 10px;
}
.cmb2-context-wrap .cmb-row:last-of-type {
  margin-bottom: 0;
}

/* one column on the post write/edit screen */
@media only screen and (max-width: 850px) {
  .cmb2-context-wrap.cmb2-context-wrap-form_top {
    margin-right: 0;
  }
}
/*--------------------------------------------------------------
 * Options page
--------------------------------------------------------------*/
.cmb2-options-page {
  max-width: 1200px;
}
.cmb2-options-page.wrap > h2 {
  margin-bottom: 1em;
}
.cmb2-options-page .cmb2-metabox > .cmb-row {
  padding: 1em;
  margin-top: -1px;
  background: #ffffff;
  border: 1px solid #e9e9e9;
  box-shadow: 0 1px 1px rgba(0, 0, 0, 0.05);
}
.cmb2-options-page .cmb2-metabox > .cmb-row > .cmb-th {
  padding: 0;
  font-weight: initial;
}
.cmb2-options-page .cmb2-metabox > .cmb-row > .cmb-th + .cmb-td {
  float: none;
  padding: 0 0 0 1em;
  margin-left: 200px;
}
@media (max-width: 450px) {
  .cmb2-options-page .cmb2-metabox > .cmb-row > .cmb-th + .cmb-td {
    padding: 0;
    margin-left: 0;
  }
}
.cmb2-options-page .cmb2-wrap .cmb-type-title {
  margin-top: 1em;
  padding: 0.6em 1em;
  background-color: #fafafa;
}
.cmb2-options-page .cmb2-wrap .cmb-type-title .cmb2-metabox-title {
  font-size: 12px;
  margin-top: 0;
  margin-bottom: 0;
  text-transform: uppercase;
}
.cmb2-options-page .cmb2-wrap .cmb-type-title .cmb2-metabox-description {
  padding-top: 0.25em;
}
.cmb2-options-page .cmb-repeatable-group .cmb-group-description .cmb-th {
  padding: 0 0 0.8em 0;
}
.cmb2-options-page .cmb-repeatable-group .cmb-group-name {
  font-size: 16px;
  margin-top: 0;
  margin-bottom: 0;
}
.cmb2-options-page .cmb-repeatable-group .cmb-th > .cmb2-metabox-description {
  font-weight: 400;
  padding-bottom: 0 !important;
}

/*--------------------------------------------------------------
 * New-Term Page
--------------------------------------------------------------*/
#addtag .cmb-th {
  float: none;
  width: auto;
  padding: 20px 0 0;
}
#addtag .cmb-td {
  padding: 0;
}
#addtag .cmb-th + .cmb-td {
  float: none;
}
#addtag select {
  max-width: 100%;
}
#addtag .cmb2-metabox {
  padding-bottom: 20px;
}
#addtag .cmb-row li label {
  display: inline;
}

/*--------------------------------------------------------------
 * Misc.
--------------------------------------------------------------*/
#poststuff .cmb-repeatable-group h2 {
  margin: 0;
}

.edit-tags-php .cmb2-metabox-title,
.profile-php .cmb2-metabox-title,
.user-edit-php .cmb2-metabox-title {
  font-size: 1.4em;
}

.cmb2-postbox .cmb-spinner, .cmb2-no-box-wrap .cmb-spinner {
  float: left;
  display: none;
}

.cmb-spinner {
  display: none;
}
.cmb-spinner.is-active {
  display: block;
}

/*--------------------------------------------------------------
 * Sidebar Placement Adjustments
--------------------------------------------------------------*/
.inner-sidebar .cmb2-wrap > .cmb-field-list > .cmb-row,
#side-sortables .cmb2-wrap > .cmb-field-list > .cmb-row {
  padding: 1.4em 0;
}
.inner-sidebar .cmb2-wrap input[type=text]:not(.wp-color-picker),
#side-sortables .cmb2-wrap input[type=text]:not(.wp-color-picker) {
  width: 100%;
}
.inner-sidebar .cmb2-wrap input + input:not(.wp-picker-clear), .inner-sidebar .cmb2-wrap input + select,
#side-sortables .cmb2-wrap input + input:not(.wp-picker-clear),
#side-sortables .cmb2-wrap input + select {
  margin-left: 0;
  margin-top: 1em;
  display: block;
}
.inner-sidebar .cmb2-wrap input.cmb2-text-money,
#side-sortables .cmb2-wrap input.cmb2-text-money {
  max-width: 70%;
}
.inner-sidebar .cmb2-wrap input.cmb2-text-money + .cmb2-metabox-description,
#side-sortables .cmb2-wrap input.cmb2-text-money + .cmb2-metabox-description {
  display: block;
}
.inner-sidebar .cmb2-wrap label,
#side-sortables .cmb2-wrap label {
  display: block;
  font-weight: 700;
  padding: 0 0 5px;
}
.inner-sidebar textarea,
#side-sortables textarea {
  max-width: 99%;
}
.inner-sidebar .cmb-repeatable-group,
#side-sortables .cmb-repeatable-group {
  border-bottom: 1px solid #e9e9e9;
}
.inner-sidebar .cmb-type-group > .cmb-td > .cmb-repeatable-group,
#side-sortables .cmb-type-group > .cmb-td > .cmb-repeatable-group {
  border-bottom: 0;
  margin-bottom: -1.4em;
}
.inner-sidebar .cmb-th,
.inner-sidebar .cmb-td:not(.cmb-remove-row),
.inner-sidebar .cmb-th + .cmb-td,
#side-sortables .cmb-th,
#side-sortables .cmb-td:not(.cmb-remove-row),
#side-sortables .cmb-th + .cmb-td {
  width: 100%;
  display: block;
  float: none;
}
.inner-sidebar .closed .inside,
#side-sortables .closed .inside {
  display: none;
}
.inner-sidebar .cmb-th,
#side-sortables .cmb-th {
  display: block;
  float: none;
  padding-bottom: 1em;
  text-align: left;
  width: 100%;
}
.inner-sidebar .cmb-th label,
#side-sortables .cmb-th label {
  display: block;
  margin-top: 0;
  margin-bottom: 0.5em;
}
.inner-sidebar .cmb-th label,
#side-sortables .cmb-th label {
  font-size: 14px;
  line-height: 1.4em;
}
.inner-sidebar .cmb-th,
#side-sortables .cmb-th {
  padding-left: 0;
  padding-right: 0;
}
.inner-sidebar .cmb-group-description .cmb-th,
#side-sortables .cmb-group-description .cmb-th {
  padding-top: 0;
}
.inner-sidebar .cmb-group-description .cmb2-metabox-description,
#side-sortables .cmb-group-description .cmb2-metabox-description {
  padding: 0;
}
.inner-sidebar .cmb-group-title .cmb-th,
#side-sortables .cmb-group-title .cmb-th {
  padding: 0;
}
.inner-sidebar .cmb-repeatable-grouping + .cmb-repeatable-grouping,
#side-sortables .cmb-repeatable-grouping + .cmb-repeatable-grouping {
  margin-top: 1em;
}
.inner-sidebar .cmb2-media-status .img-status img,
.inner-sidebar .cmb2-media-status .embed-status img,
#side-sortables .cmb2-media-status .img-status img,
#side-sortables .cmb2-media-status .embed-status img {
  max-width: 90%;
  height: auto;
}
.inner-sidebar .cmb2-list label,
#side-sortables .cmb2-list label {
  display: inline;
  font-weight: normal;
}
.inner-sidebar .cmb2-metabox-description,
#side-sortables .cmb2-metabox-description {
  display: block;
  padding: 7px 0 0;
}
.inner-sidebar .cmb-type-checkbox .cmb-td label,
.inner-sidebar .cmb-type-checkbox .cmb2-metabox-description,
#side-sortables .cmb-type-checkbox .cmb-td label,
#side-sortables .cmb-type-checkbox .cmb2-metabox-description {
  font-weight: normal;
  display: inline;
}
.inner-sidebar .cmb-row .cmb2-metabox-description,
#side-sortables .cmb-row .cmb2-metabox-description {
  padding-bottom: 1.8em;
}
.inner-sidebar .cmb2-metabox-title,
#side-sortables .cmb2-metabox-title {
  font-size: 1.2em;
  font-style: italic;
}
.inner-sidebar .cmb-remove-row,
#side-sortables .cmb-remove-row {
  clear: both;
  padding-top: 12px;
  padding-bottom: 0;
}
.inner-sidebar .cmb2-upload-button,
#side-sortables .cmb2-upload-button {
  clear: both;
  margin-top: 12px;
}

/*--------------------------------------------------------------
 * Collapsible UI
--------------------------------------------------------------*/
.cmb2-metabox .cmbhandle {
  color: #757575;
  float: right;
  width: 27px;
  height: 30px;
  cursor: pointer;
  right: -1em;
  position: relative;
}
.cmb2-metabox .cmbhandle:before {
  content: "\f142";
  right: 12px;
  font: normal 20px/1 "dashicons";
  speak: none;
  display: inline-block;
  padding: 8px 10px;
  top: 0;
  position: relative;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  text-decoration: none !important;
}
.cmb2-metabox .postbox.closed .cmbhandle:before {
  content: "\f140";
}
.cmb2-metabox button.dashicons-before.dashicons-no-alt.cmb-remove-group-row {
  -webkit-appearance: none !important;
  background: none !important;
  border: none !important;
  position: absolute;
  left: 0;
  top: 0.5em;
  line-height: 1em;
  padding: 2px 6px 3px;
  opacity: 0.5;
}
.cmb2-metabox button.dashicons-before.dashicons-no-alt.cmb-remove-group-row:not([disabled]) {
  cursor: pointer;
  color: #a00;
  opacity: 1;
}
.cmb2-metabox button.dashicons-before.dashicons-no-alt.cmb-remove-group-row:not([disabled]):hover {
  color: #f00;
}

/*
 * jQuery UI CSS Framework 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Theming/API
 *
 * WordPress Styles adopted from "jQuery UI Datepicker CSS for WordPress"
 * https://github.com/stuttter/wp-datepicker-styling
 *
 */
* html .cmb2-element.ui-helper-clearfix {
  height: 1%;
}

.cmb2-element.ui-datepicker, .cmb2-element .ui-datepicker {
  padding: 0;
  margin: 0;
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
  background-color: #fff;
  border: 1px solid #dfdfdf;
  border-top: none;
  -webkit-box-shadow: 0 3px 6px rgba(0, 0, 0, 0.075);
  box-shadow: 0 3px 6px rgba(0, 0, 0, 0.075);
  min-width: 17em;
  width: auto;
}
.cmb2-element.ui-datepicker *, .cmb2-element .ui-datepicker * {
  padding: 0;
  font-family: "Open Sans", sans-serif;
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
}
.cmb2-element.ui-datepicker table, .cmb2-element .ui-datepicker table {
  font-size: 13px;
  margin: 0;
  border: none;
  border-collapse: collapse;
}
.cmb2-element.ui-datepicker .ui-widget-header,
.cmb2-element.ui-datepicker .ui-datepicker-header, .cmb2-element .ui-datepicker .ui-widget-header,
.cmb2-element .ui-datepicker .ui-datepicker-header {
  background-image: none;
  border: none;
  color: #fff;
  font-weight: normal;
}
.cmb2-element.ui-datepicker .ui-datepicker-header .ui-state-hover, .cmb2-element .ui-datepicker .ui-datepicker-header .ui-state-hover {
  background: transparent;
  border-color: transparent;
  cursor: pointer;
}
.cmb2-element.ui-datepicker .ui-datepicker-title, .cmb2-element .ui-datepicker .ui-datepicker-title {
  margin: 0;
  padding: 10px 0;
  color: #fff;
  font-size: 14px;
  line-height: 14px;
  text-align: center;
}
.cmb2-element.ui-datepicker .ui-datepicker-title select, .cmb2-element .ui-datepicker .ui-datepicker-title select {
  margin-top: -8px;
  margin-bottom: -8px;
}
.cmb2-element.ui-datepicker .ui-datepicker-prev,
.cmb2-element.ui-datepicker .ui-datepicker-next, .cmb2-element .ui-datepicker .ui-datepicker-prev,
.cmb2-element .ui-datepicker .ui-datepicker-next {
  position: relative;
  top: 0;
  height: 34px;
  width: 34px;
}
.cmb2-element.ui-datepicker .ui-state-hover.ui-datepicker-prev,
.cmb2-element.ui-datepicker .ui-state-hover.ui-datepicker-next, .cmb2-element .ui-datepicker .ui-state-hover.ui-datepicker-prev,
.cmb2-element .ui-datepicker .ui-state-hover.ui-datepicker-next {
  border: none;
}
.cmb2-element.ui-datepicker .ui-datepicker-prev,
.cmb2-element.ui-datepicker .ui-datepicker-prev-hover, .cmb2-element .ui-datepicker .ui-datepicker-prev,
.cmb2-element .ui-datepicker .ui-datepicker-prev-hover {
  left: 0;
}
.cmb2-element.ui-datepicker .ui-datepicker-next,
.cmb2-element.ui-datepicker .ui-datepicker-next-hover, .cmb2-element .ui-datepicker .ui-datepicker-next,
.cmb2-element .ui-datepicker .ui-datepicker-next-hover {
  right: 0;
}
.cmb2-element.ui-datepicker .ui-datepicker-next span,
.cmb2-element.ui-datepicker .ui-datepicker-prev span, .cmb2-element .ui-datepicker .ui-datepicker-next span,
.cmb2-element .ui-datepicker .ui-datepicker-prev span {
  display: none;
}
.cmb2-element.ui-datepicker .ui-datepicker-prev, .cmb2-element .ui-datepicker .ui-datepicker-prev {
  float: left;
}
.cmb2-element.ui-datepicker .ui-datepicker-next, .cmb2-element .ui-datepicker .ui-datepicker-next {
  float: right;
}
.cmb2-element.ui-datepicker .ui-datepicker-prev:before,
.cmb2-element.ui-datepicker .ui-datepicker-next:before, .cmb2-element .ui-datepicker .ui-datepicker-prev:before,
.cmb2-element .ui-datepicker .ui-datepicker-next:before {
  font: normal 20px/34px "dashicons";
  padding-left: 7px;
  color: #fff;
  speak: none;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  width: 34px;
  height: 34px;
}
.cmb2-element.ui-datepicker .ui-datepicker-prev:before, .cmb2-element .ui-datepicker .ui-datepicker-prev:before {
  content: "\f341";
}
.cmb2-element.ui-datepicker .ui-datepicker-next:before, .cmb2-element .ui-datepicker .ui-datepicker-next:before {
  content: "\f345";
}
.cmb2-element.ui-datepicker .ui-datepicker-prev-hover:before,
.cmb2-element.ui-datepicker .ui-datepicker-next-hover:before, .cmb2-element .ui-datepicker .ui-datepicker-prev-hover:before,
.cmb2-element .ui-datepicker .ui-datepicker-next-hover:before {
  opacity: 0.7;
}
.cmb2-element.ui-datepicker select.ui-datepicker-month,
.cmb2-element.ui-datepicker select.ui-datepicker-year, .cmb2-element .ui-datepicker select.ui-datepicker-month,
.cmb2-element .ui-datepicker select.ui-datepicker-year {
  width: 33%;
  background: transparent;
  border-color: transparent;
  box-shadow: none;
  color: #fff;
}
.cmb2-element.ui-datepicker select.ui-datepicker-month option,
.cmb2-element.ui-datepicker select.ui-datepicker-year option, .cmb2-element .ui-datepicker select.ui-datepicker-month option,
.cmb2-element .ui-datepicker select.ui-datepicker-year option {
  color: #333;
}
.cmb2-element.ui-datepicker thead, .cmb2-element .ui-datepicker thead {
  color: #fff;
  font-weight: 600;
}
.cmb2-element.ui-datepicker thead th, .cmb2-element .ui-datepicker thead th {
  font-weight: normal;
}
.cmb2-element.ui-datepicker th, .cmb2-element .ui-datepicker th {
  padding: 10px;
}
.cmb2-element.ui-datepicker td, .cmb2-element .ui-datepicker td {
  padding: 0;
  border: 1px solid #f4f4f4;
}
.cmb2-element.ui-datepicker td.ui-datepicker-other-month, .cmb2-element .ui-datepicker td.ui-datepicker-other-month {
  border: transparent;
}
.cmb2-element.ui-datepicker td.ui-datepicker-week-end, .cmb2-element .ui-datepicker td.ui-datepicker-week-end {
  background-color: #f4f4f4;
  border: 1px solid #f4f4f4;
}
.cmb2-element.ui-datepicker td.ui-datepicker-week-end.ui-datepicker-today, .cmb2-element .ui-datepicker td.ui-datepicker-week-end.ui-datepicker-today {
  -webkit-box-shadow: inset 0px 0px 1px 0px rgba(0, 0, 0, 0.1);
  -moz-box-shadow: inset 0px 0px 1px 0px rgba(0, 0, 0, 0.1);
  box-shadow: inset 0px 0px 1px 0px rgba(0, 0, 0, 0.1);
}
.cmb2-element.ui-datepicker td.ui-datepicker-today, .cmb2-element .ui-datepicker td.ui-datepicker-today {
  background-color: #f0f0c0;
}
.cmb2-element.ui-datepicker td.ui-datepicker-current-day, .cmb2-element .ui-datepicker td.ui-datepicker-current-day {
  background: #bbdd88;
}
.cmb2-element.ui-datepicker td .ui-state-default, .cmb2-element .ui-datepicker td .ui-state-default {
  background: transparent;
  border: none;
  text-align: center;
  text-decoration: none;
  width: auto;
  display: block;
  padding: 5px 10px;
  font-weight: normal;
  color: #444;
}
.cmb2-element.ui-datepicker td.ui-state-disabled .ui-state-default, .cmb2-element .ui-datepicker td.ui-state-disabled .ui-state-default {
  opacity: 0.5;
}
.cmb2-element.ui-datepicker, .cmb2-element .ui-datepicker {
  /* Default Color Scheme */
}
.cmb2-element.ui-datepicker .ui-widget-header,
.cmb2-element.ui-datepicker .ui-datepicker-header, .cmb2-element .ui-datepicker .ui-widget-header,
.cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #00a0d2;
}
.cmb2-element.ui-datepicker thead, .cmb2-element .ui-datepicker thead {
  background: #32373c;
}
.cmb2-element.ui-datepicker td .ui-state-hover, .cmb2-element.ui-datepicker td .ui-state-active, .cmb2-element .ui-datepicker td .ui-state-hover, .cmb2-element .ui-datepicker td .ui-state-active {
  background: #0073aa;
  color: #fff;
}
.cmb2-element.ui-datepicker .ui-timepicker-div, .cmb2-element .ui-datepicker .ui-timepicker-div {
  font-size: 14px;
}
.cmb2-element.ui-datepicker .ui-timepicker-div dl, .cmb2-element .ui-datepicker .ui-timepicker-div dl {
  text-align: left;
  padding: 0 0.6em;
}
.cmb2-element.ui-datepicker .ui-timepicker-div dl dt, .cmb2-element .ui-datepicker .ui-timepicker-div dl dt {
  float: left;
  clear: left;
  padding: 0 0 0 5px;
}
.cmb2-element.ui-datepicker .ui-timepicker-div dl dd, .cmb2-element .ui-datepicker .ui-timepicker-div dl dd {
  margin: 0 10px 10px 40%;
}
.cmb2-element.ui-datepicker .ui-timepicker-div dl dd select, .cmb2-element .ui-datepicker .ui-timepicker-div dl dd select {
  width: 100%;
}
.cmb2-element.ui-datepicker .ui-timepicker-div + .ui-datepicker-buttonpane, .cmb2-element .ui-datepicker .ui-timepicker-div + .ui-datepicker-buttonpane {
  padding: 0.6em;
  text-align: left;
}
.cmb2-element.ui-datepicker .ui-timepicker-div + .ui-datepicker-buttonpane .button-primary, .cmb2-element.ui-datepicker .ui-timepicker-div + .ui-datepicker-buttonpane .button-secondary, .cmb2-element .ui-datepicker .ui-timepicker-div + .ui-datepicker-buttonpane .button-primary, .cmb2-element .ui-datepicker .ui-timepicker-div + .ui-datepicker-buttonpane .button-secondary {
  padding: 0 10px 1px;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
  margin: 0 0.6em 0.4em 0.4em;
}

.admin-color-fresh .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-fresh .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-fresh .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-fresh .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #00a0d2;
}
.admin-color-fresh .cmb2-element.ui-datepicker thead, .admin-color-fresh .cmb2-element .ui-datepicker thead {
  background: #32373c;
}
.admin-color-fresh .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-fresh .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #0073aa;
  color: #fff;
}

.admin-color-blue .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-blue .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-blue .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-blue .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #52accc;
}
.admin-color-blue .cmb2-element.ui-datepicker thead, .admin-color-blue .cmb2-element .ui-datepicker thead {
  background: #4796b3;
}
.admin-color-blue .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-blue .cmb2-element.ui-datepicker td .ui-state-active, .admin-color-blue .cmb2-element .ui-datepicker td .ui-state-hover, .admin-color-blue .cmb2-element .ui-datepicker td .ui-state-active {
  background: #096484;
  color: #fff;
}
.admin-color-blue .cmb2-element.ui-datepicker td.ui-datepicker-today, .admin-color-blue .cmb2-element .ui-datepicker td.ui-datepicker-today {
  background: #eee;
}

.admin-color-coffee .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-coffee .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-coffee .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-coffee .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #59524c;
}
.admin-color-coffee .cmb2-element.ui-datepicker thead, .admin-color-coffee .cmb2-element .ui-datepicker thead {
  background: #46403c;
}
.admin-color-coffee .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-coffee .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #c7a589;
  color: #fff;
}

.admin-color-ectoplasm .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-ectoplasm .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-ectoplasm .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-ectoplasm .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #523f6d;
}
.admin-color-ectoplasm .cmb2-element.ui-datepicker thead, .admin-color-ectoplasm .cmb2-element .ui-datepicker thead {
  background: #413256;
}
.admin-color-ectoplasm .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-ectoplasm .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #a3b745;
  color: #fff;
}

.admin-color-midnight .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-midnight .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-midnight .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-midnight .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #363b3f;
}
.admin-color-midnight .cmb2-element.ui-datepicker thead, .admin-color-midnight .cmb2-element .ui-datepicker thead {
  background: #26292c;
}
.admin-color-midnight .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-midnight .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #e14d43;
  color: #fff;
}

.admin-color-ocean .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-ocean .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-ocean .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-ocean .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #738e96;
}
.admin-color-ocean .cmb2-element.ui-datepicker thead, .admin-color-ocean .cmb2-element .ui-datepicker thead {
  background: #627c83;
}
.admin-color-ocean .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-ocean .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #9ebaa0;
  color: #fff;
}

.admin-color-sunrise .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-sunrise .cmb2-element.ui-datepicker .ui-datepicker-header,
.admin-color-sunrise .cmb2-element.ui-datepicker .ui-datepicker-header .ui-state-hover, .admin-color-sunrise .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-sunrise .cmb2-element .ui-datepicker .ui-datepicker-header,
.admin-color-sunrise .cmb2-element .ui-datepicker .ui-datepicker-header .ui-state-hover {
  background: #cf4944;
}
.admin-color-sunrise .cmb2-element.ui-datepicker th, .admin-color-sunrise .cmb2-element .ui-datepicker th {
  border-color: #be3631;
  background: #be3631;
}
.admin-color-sunrise .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-sunrise .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #dd823b;
  color: #fff;
}

.admin-color-light .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-light .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-light .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-light .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #e5e5e5;
}
.admin-color-light .cmb2-element.ui-datepicker select.ui-datepicker-month,
.admin-color-light .cmb2-element.ui-datepicker select.ui-datepicker-year, .admin-color-light .cmb2-element .ui-datepicker select.ui-datepicker-month,
.admin-color-light .cmb2-element .ui-datepicker select.ui-datepicker-year {
  color: #555;
}
.admin-color-light .cmb2-element.ui-datepicker thead, .admin-color-light .cmb2-element .ui-datepicker thead {
  background: #888;
}
.admin-color-light .cmb2-element.ui-datepicker .ui-datepicker-title,
.admin-color-light .cmb2-element.ui-datepicker td .ui-state-default,
.admin-color-light .cmb2-element.ui-datepicker .ui-datepicker-prev:before,
.admin-color-light .cmb2-element.ui-datepicker .ui-datepicker-next:before, .admin-color-light .cmb2-element .ui-datepicker .ui-datepicker-title,
.admin-color-light .cmb2-element .ui-datepicker td .ui-state-default,
.admin-color-light .cmb2-element .ui-datepicker .ui-datepicker-prev:before,
.admin-color-light .cmb2-element .ui-datepicker .ui-datepicker-next:before {
  color: #555;
}
.admin-color-light .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-light .cmb2-element.ui-datepicker td .ui-state-active, .admin-color-light .cmb2-element .ui-datepicker td .ui-state-hover, .admin-color-light .cmb2-element .ui-datepicker td .ui-state-active {
  background: #ccc;
}
.admin-color-light .cmb2-element.ui-datepicker td.ui-datepicker-today, .admin-color-light .cmb2-element .ui-datepicker td.ui-datepicker-today {
  background: #eee;
}

.admin-color-bbp-evergreen .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-bbp-evergreen .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-bbp-evergreen .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-bbp-evergreen .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #56b274;
}
.admin-color-bbp-evergreen .cmb2-element.ui-datepicker thead, .admin-color-bbp-evergreen .cmb2-element .ui-datepicker thead {
  background: #36533f;
}
.admin-color-bbp-evergreen .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-bbp-evergreen .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #446950;
  color: #fff;
}

.admin-color-bbp-mint .cmb2-element.ui-datepicker .ui-widget-header,
.admin-color-bbp-mint .cmb2-element.ui-datepicker .ui-datepicker-header, .admin-color-bbp-mint .cmb2-element .ui-datepicker .ui-widget-header,
.admin-color-bbp-mint .cmb2-element .ui-datepicker .ui-datepicker-header {
  background: #4ca26a;
}
.admin-color-bbp-mint .cmb2-element.ui-datepicker thead, .admin-color-bbp-mint .cmb2-element .ui-datepicker thead {
  background: #4f6d59;
}
.admin-color-bbp-mint .cmb2-element.ui-datepicker td .ui-state-hover, .admin-color-bbp-mint .cmb2-element .ui-datepicker td .ui-state-hover {
  background: #5fb37c;
  color: #fff;
}

/*--------------------------------------------------------------
 * Character counter
--------------------------------------------------------------*/
.cmb2-char-counter-wrap {
  margin: 0.5em 0 1em;
}
.cmb2-char-counter-wrap input[type=text] {
  font-size: 12px;
  width: 25px;
}
.cmb2-char-counter-wrap.cmb2-max-exceeded input[type=text] {
  border-color: #a00 !important;
}
.cmb2-char-counter-wrap.cmb2-max-exceeded .cmb2-char-max-msg {
  display: inline-block;
}

.cmb2-char-max-msg {
  color: #a00;
  display: none;
  font-weight: 600;
  margin-left: 1em;
}
