<?php return array(
    'root' => array(
        'name' => 'codeinwp/redirection-for-contact-form-7',
        'pretty_version' => 'v3.2.10',
        'version' => '3.2.10.0',
        'reference' => 'afa64e9b8adc274e8a3b3836836694dc1ebbce41',
        'type' => 'wordpress-plugin',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'codeinwp/redirection-for-contact-form-7' => array(
            'pretty_version' => 'v3.2.10',
            'version' => '3.2.10.0',
            'reference' => 'afa64e9b8adc274e8a3b3836836694dc1ebbce41',
            'type' => 'wordpress-plugin',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'codeinwp/themeisle-sdk' => array(
            'pretty_version' => '3.3.51',
            'version' => '3.3.51.0',
            'reference' => 'bb2a8414b0418b18c68c9ff1df3d7fb10467928d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../codeinwp/themeisle-sdk',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
