<?php
return ['incredibile','sorprendente','meraviglioso','unico','bello','felicità','brillante','affascinante','carismatico','scioccante','chiaro','completamente','riservato','fiducia','significativo','creativo','definitivamente','delizioso','dimostrare','sbrigati','determinato','degno','dinamico','impressionante','essenziale','ispiratore','innovativo','intenso','efficace','magico','magnifico','storico','importante','incredibile','indispensabile','indimenticabile','irresistibile','leggendario','luminoso','lusso','magico','magnifico','maestoso','memorabile','miracoloso','motivante','necessario','nuovo','ufficiale','perfetto','appassionato','persuasivo','fenomenale','piacere','popolare','potere','prestigioso','prodigioso','profondo','prospero','potente','qualità','radiante','veloce','di successo','rivoluzionario','soddisfatto','sicurezza','sensazionale','sereno','lussuoso','splendido','sublime','sorprendente','talentuoso','terrificante','unico','valore','vibrante','vittorioso','vivace','veramente','zelante','autentico','avventuroso','spettacolare','esclusivo','garantito','straordinario','favoloso','affascinante','formidabile','geniale','grandioso','gratuito','abile','illimitato','impeccabile','infallibile','infinitamente','influente','ingegnoso','indimenticabile','insostituibile','leader','maestro','notevole','innovativo','pioniere','potente','riconosciuto','rivoluzionario','sorprendente','superiore','trionfante','ultra','coraggioso','prezioso','avanguardia','vigoroso','visionario','volontà','vitale','trionfo','glorioso','inarrestabile','ineguagliabile','intelligente','invincibile','libertà','orgoglio','pace','progresso','rinnovato','saggezza','soddisfazione','sicuro','serenità','superamento','talento','trascendente','trasformativo','coraggio','vittoria'];
